# -*- coding: utf-8 -*-
from plone.app.contenttypes.browser.link_redirect_view import LinkRedirectView as LinkRedirect
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class LinkRedirectView(LinkRedirect):

    index = ViewPageTemplateFile('link_redirect.pt')

    def __call__(self):
        if not self.context.remoteUrl:
            return self.index()
        return super().__call__()



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IColourField
from zope.interface import implementer
from zope.schema import TextLine


@implementer(IColourField)
class ColourField(TextLine):
    """ Colour Field
    """



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.fields.colour import ColourField
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IColourBehaviorSchema(model.Schema):
    """ Colour Behavior Schema
    """

    colour = ColourField(
        title=_('Colour'),
        required=False,
        default='#000000',
    )


@implementer(IColourBehaviorSchema)
@adapter(IDexterityContent)
class ColourBehaviorFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def colour(self):
        return self.context.colour

    @colour.setter
    def colour(self, value):
        self.context.colour = value



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.fields.icon import IconField
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IIconBehaviorSchema(model.Schema):
    """ Icon Behavior Schema
    """

    icon = IconField(
        title=_('Icon'),
        description=_('Select an icon'),
        required=True,
    )


@implementer(IIconBehaviorSchema)
@adapter(IDexterityContent)
class IconBehaviorFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def icon(self):
        return self.context.icon

    @icon.setter
    def icon(self, value):
        self.context.icon = value



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.widgets.osmap import OSMapWidget
from plone.app.registry.browser import controlpanel
from plone.autoform import directives
from zope import schema
from zope.interface import Interface


class ICoordinatesControlPanel(Interface):

    directives.widget('map_center', OSMapWidget)
    map_center = schema.TextLine(
        title=_('Map center'),
        description=_('Set the default coordinates for the map widget.'),
        default='0|0',
    )


class CoordinatesControlPanelForm(controlpanel.RegistryEditForm):
    """ CoordinatesControlPanelForm
    """
    schema = ICoordinatesControlPanel
    schema_prefix = 'coordinates'
    label = _('Coordinates')


class CoordinatesControlPanel(controlpanel.ControlPanelFormWrapper):
    """ CoordinatesControlPanel
    """
    form = CoordinatesControlPanelForm



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.interfaces import IIconField
from contenttypes.basic.interfaces import IIconWidget
from lxml import etree
from plone import api
from z3c.form.browser.text import TextWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer


@implementer(IIconWidget)
class IconWidget(TextWidget):

    name = 'icon-widget'
    label = _('Icon Widget')
    _xml = None

    @property
    def portal_url(self):
        return api.portal.get().absolute_url()

    @property
    def xml(self):
        if not self._xml:
            self._xml = self.get_xml()
        return self._xml

    def get_xml(self):
        sprite_view = api.portal.get().restrictedTraverse('@@sprite-icons')
        result = sprite_view()
        symbols = []

        root = etree.fromstring(result)
        for element in root.iter('{http://www.w3.org/2000/svg}symbol'):
            symbols.append(element.get('id'))

        return symbols


@implementer(IFieldWidget)
@adapter(IIconField, IFormLayer)
def IconFieldWidget(field, request):
    return FieldWidget(field, IconWidget(request))



# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import ILink
from plone.supermodel import model
from z3c.form.interfaces import ITextWidget
from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from zope.schema.interfaces import ITextLine


class IContenttypesBasicLayer(IDefaultBrowserLayer):
    """Marker interface that defines a browser layer."""


class IAnnouncement(model.Schema):
    """ Marker interface and Schema for Announcement
    """


class IBanner(model.Schema):
    """ Marker interface and Schema for Banner
    """


class IIcon(model.Schema):
    """ Marker interface and Schema for Icon
    """


class ISlide(model.Schema):
    """ Marker interface and Schema for Slide
    """


class IHideTextBehavior(Interface):
    """ Marker interface for Text Info Behavior
    """


class IButtonTextBehavior(Interface):
    """ Marker interface for Button Text Behavior
    """


class ITextPositionBehavior(Interface):
    """ Marker interface for Text Position Behavior
    """


class IColourBehavior(Interface):
    """ Marker interface for Colour Behavior
    """


class IIconBehavior(Interface):
    """ Marker interface for Icon Behavior
    """


class ILinkBehavior(ILink):
    """ Marker interface for Link Behavior
    """


class IOSMapBehavior(Interface):
    """ Marker interface for OSMap Behavior
    """


class IIconWidget(ITextWidget):
    """ Marker interface for Icon Widget
    """


class IColourWidget(ITextWidget):
    """ Marker interface for Colour Widget
    """


class IOSMapWidget(ITextWidget):
    """ Marker interface for OSMap Widget
    """


class IIconField(ITextLine):
    """ Marker interface for Icon Field
    """


class IColourField(ITextLine):
    """ Marker interface for Colour Field
    """


class IOSMapField(ITextLine):
    """ Marker interface for OSMap Field
    """



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from contenttypes.basic.interfaces import IOSMapField
from contenttypes.basic.interfaces import IOSMapWidget
from datetime import datetime
from plone import api
from plone.api.exc import InvalidParameterError
from z3c.form.browser.text import TextWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.interface import implementer


@implementer(IOSMapWidget)
class OSMapWidget(TextWidget):

    name = 'osmap-widget'
    label = _('OSMap Widget')
    timestamp = datetime.now().strftime('%s')

    @property
    def portal_url(self):
        return api.portal.get().absolute_url()

    @property
    def default_value(self):
        return self.field.default or '0|0'

    def default_value_map(self):
        try:
            return api.portal.get_registry_record(name='coordinates.map_center')
        except InvalidParameterError:
            return '0|0'


@implementer(IFieldWidget)
@adapter(IOSMapField, IFormLayer)
def OSMapFieldWidget(field, request):
    return FieldWidget(field, OSMapWidget(request))


@adapter(IOSMapField, IOSMapWidget)
class OSMapConverter(BaseDataConverter):
    """ Convert between the context and the widget
    """

    def toWidgetValue(self, value):
        return self.parse_coordinates(value)

    def toFieldValue(self, value):
        return self.parse_coordinates(value)

    def parse_coordinates(self, value):
        if not value:
            return '0|0'

        coordinates = value.split('|')
        if len(coordinates) != 2:
            return '0|0'

        coordinates = map(lambda coord: self.parse_float(coord), coordinates)
        return '|'.join(coordinates)

    def parse_float(self, value):
        try:
            value = float(value)
            value = str(value)
            if value == '0.0':
                value = '0'
            return value
        except ValueError:
            return '0'



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IIconField
from zope.interface import implementer
from zope.schema import TextLine


@implementer(IIconField)
class IconField(TextLine):
    """ Icon Field
    """



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class IHideTextSchema(model.Schema):
    """ Text Info Schema
    """

    hide_text = schema.Bool(
        title=_('Hide Text'),
        required=False,
    )


@implementer(IHideTextSchema)
@adapter(IDexterityContent)
class HideTextFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def hide_text(self):
        return self.context.hide_text

    @hide_text.setter
    def hide_text(self, value):
        self.context.hide_text = value



# -*- coding: utf-8 -*-
from contenttypes.basic.interfaces import IButtonTextBehavior
from contenttypes.basic.interfaces import IColourBehavior
from contenttypes.basic.interfaces import IHideTextBehavior
from contenttypes.basic.interfaces import IIconBehavior
from contenttypes.basic.interfaces import ILinkBehavior
from contenttypes.basic.interfaces import IOSMapBehavior
from contenttypes.basic.interfaces import ITextPositionBehavior
from contenttypes.basic.testing import CONTENTTYPES_BASIC_FUNCTIONAL_TESTING
from contenttypes.basic.tests import TestUtils
from plone.app.contenttypes.interfaces import ILink
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.uuid.utils import uuidToCatalogBrain


class BehaviorFunctionalTest(TestUtils):

    layer = CONTENTTYPES_BASIC_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_behavior_hide_text(self):
        self._create_dummy_fti(behavior='contenttypes.basic.hide_text')
        self._create_obj()
        self._set_browser()

        self.assertTrue(IHideTextBehavior.providedBy(self.obj))

        self.browser.open(self.portal_url + '/test/edit')
        checkbox = self.browser.getControl(
            name='form.widgets.IHideTextSchema.hide_text:list',
        )
        checkbox.value = ['selected']
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertTrue(self.obj.hide_text)

    def test_behavior_button_text(self):
        self._create_dummy_fti(behavior='contenttypes.basic.button_text')
        self._create_obj()
        self._set_browser()

        self.assertTrue(IButtonTextBehavior.providedBy(self.obj))
        self.assertEqual(self.obj.button_text, 'More information')

        self.browser.open(self.portal_url + '/test/edit')
        textline = self.browser.getControl(
            name='form.widgets.IButtonTextSchema.button_text',
        )
        textline_value = 'See more'
        textline.value = textline_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertEqual(self.obj.button_text, textline_value)

    def test_behavior_text_position(self):
        self._create_dummy_fti(behavior='contenttypes.basic.text_position')
        self._create_obj()
        self._set_browser()

        self.assertTrue(ITextPositionBehavior.providedBy(self.obj))

        self.browser.open(self.portal_url + '/test/edit')
        select = self.browser.getControl(
            name='form.widgets.ITextPositionSchema.text_position:list',
        )
        select.value = ['1']
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertTrue(self.obj.text_position, 1)

    def test_behavior_icon(self):
        self._create_dummy_fti(behavior='contenttypes.basic.icon')
        self._create_obj()
        self._set_browser()

        self.assertTrue(IIconBehavior.providedBy(self.obj))

        self.browser.open(self.portal_url + '/test/edit')
        icon = self.browser.getControl(
            name='form.widgets.IIconBehaviorSchema.icon',
        )
        icon_value = 'euro'
        icon.value = icon_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertTrue(self.obj.icon, icon_value)

    def test_behavior_colour(self):
        self._create_dummy_fti(behavior='contenttypes.basic.colour')
        self._create_obj()
        self._set_browser()

        self.assertTrue(IColourBehavior.providedBy(self.obj))
        self.assertEqual(self.obj.colour, '#000000')

        self.browser.open(self.portal_url + '/test/edit')
        colour = self.browser.getControl(
            name='form.widgets.IColourBehaviorSchema.colour',
        )
        colour_value = '#ffffff'
        colour.value = colour_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertTrue(self.obj.colour, colour_value)

    def test_behavior_link(self):
        self._create_dummy_fti(behavior='contenttypes.basic.link')
        self._create_obj()
        self._set_browser()

        self.assertTrue(ILinkBehavior.providedBy(self.obj))
        self.assertTrue(ILink.providedBy(self.obj))

        uuid_link = self.obj.UID()

        self.browser.open(self.portal_url + '/test/edit')
        link = self.browser.getControl(
            name='form.widgets.ILinkBehaviorSchema.remoteUrl.internal',
        )
        link.value = uuid_link
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        brain = uuidToCatalogBrain(uuid_link)

        self.assertTrue(
            self.obj.remoteUrl,
            '{{portal_url}}/resolveuid/{uuid}'.format(uuid=uuid_link),
        )
        self.assertTrue(
            brain.getRemoteUrl,
            '/{portal_id}/resolveuid/{uuid}'.format(
                portal_id=self.portal.id,
                uuid=uuid_link,
            ),
        )

    def test_behavior_osmap(self):
        self._create_dummy_fti(behavior='contenttypes.basic.osmap')
        self._create_obj()
        self._set_browser()

        self.assertTrue(IOSMapBehavior.providedBy(self.obj))
        self.assertEqual(self.obj.geolocation, None)

        self.browser.open(self.portal_url + '/test/edit')
        geolocation = self.browser.getControl(
            name='form.widgets.IOSMapBehaviorSchema.geolocation',
        )
        geolocation_value = '41.6147605|0.6267842'
        geolocation.value = geolocation_value
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.assertTrue(self.obj.geolocation, geolocation_value)



# -*- coding: utf-8 -*-
from contenttypes.basic import _
from plone.autoform.directives import widget
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ILinkBehaviorSchema(model.Schema):
    """ Link Behavior Schema
    """

    widget(remoteUrl='plone.app.z3cform.widget.LinkFieldWidget')
    remoteUrl = schema.TextLine(
        title=_('Link'),
        required=False,
    )


@implementer(ILinkBehaviorSchema)
@adapter(IDexterityContent)
class LinkBehaviorFactory(object):

    def __init__(self, context):
        self.context = context

    @property
    def remoteUrl(self):
        return self.context.remoteUrl

    @remoteUrl.setter
    def remoteUrl(self, value):
        self.context.remoteUrl = value



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.dexterity.fti import DexterityFTI
from plone.testing.z2 import Browser

import transaction
import unittest


class TestUtils(unittest.TestCase):

    def _create_dummy_fti(self, behavior=None):
        self.fti = DexterityFTI('test')
        self.portal.portal_types._setObject('test', self.fti)
        self.fti.klass = 'plone.dexterity.content.Container'
        self.fti.filter_content_types = False
        if behavior:
            self.fti.behaviors = (behavior,)

    def _create_obj(self, portal_type='test'):
        api.content.create(
            container=self.portal,
            type=portal_type,
            id='test',
            title='Test',
        )
        transaction.commit()
        self.obj = self.portal['test']

    def _set_browser(self):
        self.app = self.layer['app']
        self.browser = Browser(self.app)
        self.browser.handleErrors = False
        self.browser.addHeader(
            'Authorization',
            'Basic {0}:{1}'.format(
                SITE_OWNER_NAME, SITE_OWNER_PASSWORD,
            ),
        )



# -*- coding: utf-8 -*-
from Products.Five import BrowserView


class SpriteView(BrowserView):
    """ SpriteView
    """

    def __call__(self):
        response = self.request.response
        response.setHeader('Content-type', 'image/svg+xml')
        return super(SpriteView, self).__call__()



# -*- coding: utf-8 -*-
from contenttypes.basic.testing import CONTENTTYPES_BASIC_FUNCTIONAL_TESTING
from contenttypes.basic.tests import TestUtils
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID


class ViewFunctionalTest(TestUtils):

    layer = CONTENTTYPES_BASIC_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_redirect_custom_view(self):
        self._create_dummy_fti(behavior='contenttypes.basic.link')
        self._create_obj()
        self._set_browser()

        self.browser.open(self.portal_url + '/test/@@link_redirect_custom_view')
        contents = self.browser.contents
        template_class = 'template-link_redirect_custom_view'
        self.assertTrue(template_class in contents)

        self.browser.open(self.portal_url + '/test/edit')
        link = self.browser.getControl(
            name='form.widgets.ILinkBehaviorSchema.remoteUrl.external',
        )
        link.value = 'https://www.google.es'
        save = self.browser.getControl(name='form.buttons.save')
        save.click()

        self.browser.open(self.portal_url + '/test/@@link_redirect_custom_view')
        contents = self.browser.contents
        message_info = 'portalMessage info'
        self.assertTrue(message_info in contents)



# -*- coding: utf-8 -*-

addAnnouncement = 'contenttypes.basic: Add Announcement'
addBanner = 'contenttypes.basic: Add Banner'
addIcon = 'contenttypes.basic: Add Icon'
addSlide = 'contenttypes.basic: Add Slide'

manageCoordinatesControlPanel = 'contenttypes.basic: Manage Coordinates ControlPanel'
